import { Injectable } from '@nestjs/common';
import { Request } from 'express';

@Injectable()
export class FormUtilityService {
  public emailRegex =
    /^[-!#$%&'*+\/0-9=?A-Z^_a-z`{|}~](\.?[-!#$%&'*+\/0-9=?A-Z^_a-z`{|}~])*@[a-zA-Z0-9](-*\.?[a-zA-Z0-9])*\.[a-zA-Z](-?[a-zA-Z0-9])+$/;
  public passwordRegex = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)[a-zA-Z\d\w\W]{8,}$/;
  public usernameRegex = /^[a-zA-Z0-9_\-.]{3,26}$/;

  /**
   * Merge array of objects into single object
   * @param flash Array of objects
   * @returns Merged object
   */
  public mergeObjectArray(flash: Record<string, any>[]): Record<string, any> {
    return flash.reduce<Record<string, any>>(
      (obj, item) => ({ ...obj, ...item }),
      {},
    );
  }

  /**
   * Trim values of specified keys of an object
   * @param entry Object
   * @param fields Fields to strip spaces from
   * @returns Object with trimmed values
   */
  public trimmed<T>(entry: T, fields: string[]): T {
    return fields.reduce<T>(
      (object, key) => ({ ...object, [key]: object[key]?.trim() }),
      entry,
    );
  }

  /**
   * Include CSRF token, messages and prefilled form values for a template with a form
   * @param req Express request
   * @param additional Additional data to pass
   * @returns Template locals
   */
  public populateTemplate(
    req: Request,
    additional: Record<string, any> = {},
  ): Record<string, any> {
    const message = req.flash('message')[0] || {};
    const form = this.mergeObjectArray(
      (req.flash('form') as Record<string, any>[]) || [],
    );

    return {
      path: req.originalUrl,
      csrf: req.csrfToken(),
      message,
      form,
      ...additional,
    };
  }
}
